@@ -3,17 +3,25 @@
 #  authz_tests.py:  testing authentication.
 #
 #  Subversion is a tool for revision control.
-#  See http://subversion.tigris.org for more information.
+#  See http://subversion.apache.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+#    Licensed to the Apache Software Foundation (ASF) under one
+#    or more contributor license agreements.  See the NOTICE file
+#    distributed with this work for additional information
+#    regarding copyright ownership.  The ASF licenses this file
+#    to you under the Apache License, Version 2.0 (the
+#    "License"); you may not use this file except in compliance
+#    with the License.  You may obtain a copy of the License at
 #
-# This software is licensed as described in the file COPYING, which
-# you should have received as part of this distribution.  The terms
-# are also available at http://subversion.tigris.org/license-1.html.
-# If newer versions of this license are posted there, you may use a
-# newer version instead, at your option.
+#      http://www.apache.org/licenses/LICENSE-2.0
 #
+#    Unless required by applicable law or agreed to in writing,
+#    software distributed under the License is distributed on an
+#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+#    KIND, either express or implied.  See the License for the
+#    specific language governing permissions and limitations
+#    under the License.
 ######################################################################
 
 # General modules
@@ -25,12 +33,18 @@ import svntest
 from svntest.main import write_restrictive_svnserve_conf
 from svntest.main import write_authz_file
 from svntest.main import server_authz_has_aliases
+from upgrade_tests import (replace_sbox_with_tarfile,
+                           replace_sbox_repo_with_tarfile,
+                           wc_is_too_old_regex)
 
 # (abbreviation)
 Item = svntest.wc.StateItem
-XFail = svntest.testcase.XFail
-Skip = svntest.testcase.Skip
-SkipUnless = svntest.testcase.SkipUnless
+Skip = svntest.testcase.Skip_deco
+SkipUnless = svntest.testcase.SkipUnless_deco
+XFail = svntest.testcase.XFail_deco
+Issues = svntest.testcase.Issues_deco
+Issue = svntest.testcase.Issue_deco
+Wimp = svntest.testcase.Wimp_deco
 
 ######################################################################
 # Tests
@@ -41,7 +55,8 @@ SkipUnless = svntest.testcase.SkipUnless
 #----------------------------------------------------------------------
 
 # regression test for issue #2486 - part 1: open_root
-
+@Issue(2486)
+@Skip(svntest.main.is_ra_type_file)
 def authz_open_root(sbox):
   "authz issue #2486 - open root"
 
@@ -73,7 +88,8 @@ def authz_open_root(sbox):
 #----------------------------------------------------------------------
 
 # regression test for issue #2486 - part 2: open_directory
-
+@Issue(2486)
+@Skip(svntest.main.is_ra_type_file)
 def authz_open_directory(sbox):
   "authz issue #2486 - open directory"
 
@@ -106,6 +122,7 @@ def authz_open_directory(sbox):
                                         None,
                                         wc_dir)
 
+@Skip(svntest.main.is_ra_type_file)
 def broken_authz_file(sbox):
   "broken authz files cause errors"
 
@@ -127,6 +144,7 @@ def broken_authz_file(sbox):
     raise svntest.verify.SVNUnexpectedStderr("Missing stderr")
 
 # test whether read access is correctly granted and denied
+@Skip(svntest.main.is_ra_type_file)
 def authz_read_access(sbox):
   "test authz for read operations"
 
@@ -141,28 +159,29 @@ def authz_read_access(sbox):
   iota_url = root_url + '/iota'
   lambda_url = B_url + '/lambda'
   alpha_url = E_url + '/alpha'
+  F_alpha_url = B_url + '/F/alpha'
   D_url = A_url + '/D'
   G_url = D_url + '/G'
   pi_url = G_url + '/pi'
   H_url = D_url + '/H'
   chi_url = H_url + '/chi'
+  fws_url = B_url + '/folder with spaces'
+  fws_empty_folder_url = fws_url + '/empty folder'
 
   if sbox.repo_url.startswith("http"):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   # create some folders with spaces in their names
-  svntest.actions.run_and_verify_svn(None, None, [],
-                                     'mkdir',
-                                     '-m', 'logmsg',
-                                     B_url+'/folder with spaces',
-                                     B_url+'/folder with spaces/empty folder')
+  svntest.actions.run_and_verify_svn(None, None, [], 'mkdir', '-m', 'logmsg',
+                                     fws_url, fws_empty_folder_url)
 
   write_restrictive_svnserve_conf(sbox.repo_dir)
 
   write_authz_file(sbox, { "/": "* = r",
                            "/A/B": "* =",
+                           "/A/B/F": "* = rw",
                            "/A/D": "* = rw",
                            "/A/D/G": ("* = rw\n" +
                                       svntest.main.wc_author + " ="),
@@ -211,7 +230,7 @@ def authz_read_access(sbox):
   # open a remote folder(ls) with spaces, should succeed
   svntest.actions.run_and_verify_svn(None,
                                      None, [], 'ls',
-                                     B_url+'/folder with spaces/empty folder')
+                                     fws_empty_folder_url)
 
   # open a remote folder(ls), unreadable through recursion: should fail
   svntest.actions.run_and_verify_svn(None,
@@ -243,7 +262,27 @@ def authz_read_access(sbox):
                                      '-m', 'logmsg',
                                      E_url, D_url)
 
+  # move a remote file, source/target ancestor is readonly: should fail
+  #
+  # Note: interesting, we deem it okay for someone to break this move
+  # into two operations, a committed copy followed by a committed
+  # deletion.  But the editor drive required to do these atomically
+  # today is prohibitive.
+  svntest.actions.run_and_verify_svn(None,
+                                     None, expected_err,
+                                     'mv', '-m', 'logmsg',
+                                     alpha_url, F_alpha_url)
+
+  ## copy a remote file, source/target ancestor is readonly
+  ## we fail here due to issue #3242.
+  #svntest.actions.run_and_verify_svn(None,
+  #                                   None, [],
+  #                                   'cp', '-m', 'logmsg',
+  #                                   alpha_url, F_alpha_url)
+
+
 # test whether write access is correctly granted and denied
+@Skip(svntest.main.is_ra_type_file)
 def authz_write_access(sbox):
   "test authz for write operations"
 
@@ -254,7 +293,7 @@ def authz_write_access(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Access denied.*"
+    expected_err = ".*svn: E220004: Access denied.*"
 
   write_authz_file(sbox, { "/": "* = r",
                            "/A/B": "* = rw",
@@ -335,6 +374,7 @@ def authz_write_access(sbox):
 
 #----------------------------------------------------------------------
 
+@Skip(svntest.main.is_ra_type_file)
 def authz_checkout_test(sbox):
   "test authz for checkout"
 
@@ -349,7 +389,7 @@ def authz_checkout_test(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   write_authz_file(sbox, { "/": "* ="})
 
@@ -373,6 +413,7 @@ def authz_checkout_test(sbox):
                           expected_output,
                           expected_wc)
 
+@Skip(svntest.main.is_ra_type_file)
 def authz_checkout_and_update_test(sbox):
   "test authz for checkout and update"
 
@@ -384,20 +425,22 @@ def authz_checkout_and_update_test(sbox):
   # 1st part: disable read access on folder A/B, checkout should not
   # download this folder
 
-  # write an authz file with *= on /A/B
+  # write an authz file with *= on /A/B and /A/mu.
   write_authz_file(sbox, { "/": "* = r",
-                           "/A/B": "* ="})
+                           "/A/B": "* =",
+                           "/A/mu": "* =",
+                           })
 
-  # checkout a working copy, should not dl /A/B
+  # checkout a working copy, should not dl /A/B or /A/mu.
   expected_output = svntest.main.greek_state.copy()
   expected_output.wc_dir = local_dir
   expected_output.tweak(status='A ', contents=None)
   expected_output.remove('A/B', 'A/B/lambda', 'A/B/E', 'A/B/E/alpha',
-                         'A/B/E/beta', 'A/B/F')
+                         'A/B/E/beta', 'A/B/F', 'A/mu')
 
   expected_wc = svntest.main.greek_state.copy()
   expected_wc.remove('A/B', 'A/B/lambda', 'A/B/E', 'A/B/E/alpha',
-                     'A/B/E/beta', 'A/B/F')
+                     'A/B/E/beta', 'A/B/F', 'A/mu')
 
   svntest.actions.run_and_verify_checkout(sbox.repo_url, local_dir,
                                           expected_output,
@@ -405,8 +448,10 @@ def authz_checkout_and_update_test(sbox):
 
   # 2nd part: now enable read access
 
-  # write an authz file with *=r on /
-  write_authz_file(sbox, { "/": "* = r"})
+  # write an authz file with *=r on /. continue to exclude mu.
+  write_authz_file(sbox, { "/": "* = r",
+                           "/A/mu": "* =",
+                           })
 
   # update the working copy, should download /A/B because we now have read
   # access
@@ -419,8 +464,10 @@ def authz_checkout_and_update_test(sbox):
     'A/B/F' : Item(status='A '),
     })
 
-  expected_wc = svntest.main.greek_state
+  expected_wc = svntest.main.greek_state.copy()
+  expected_wc.remove('A/mu')
   expected_status = svntest.actions.get_virginal_state(local_dir, 1)
+  expected_status.remove('A/mu')
 
   svntest.actions.run_and_verify_update(local_dir,
                                         expected_output,
@@ -430,6 +477,7 @@ def authz_checkout_and_update_test(sbox):
                                         None, None,
                                         None, None, 1)
 
+@Skip(svntest.main.is_ra_type_file)
 def authz_partial_export_test(sbox):
   "test authz for export with unreadable subfolder"
 
@@ -465,6 +513,7 @@ def authz_partial_export_test(sbox):
 
 #----------------------------------------------------------------------
 
+@Skip(svntest.main.is_ra_type_file)
 def authz_log_and_tracing_test(sbox):
   "test authz for log and tracing path changes"
 
@@ -477,7 +526,7 @@ def authz_log_and_tracing_test(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   write_authz_file(sbox, { "/": "* = rw\n" })
 
@@ -508,7 +557,7 @@ def authz_log_and_tracing_test(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   authz = { "/": "* = rw",
             "/A/D/G": "* ="}
@@ -527,7 +576,7 @@ def authz_log_and_tracing_test(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err2 = expected_err
   else:
-    expected_err2 = ".*svn: Item is not readable.*"
+    expected_err2 = ".*svn: E220001: Item is not readable.*"
 
   # if we do the same thing directly on the unreadable file, we get:
   # svn: Item is not readable
@@ -566,7 +615,7 @@ def authz_log_and_tracing_test(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err2 = expected_err
   else:
-    expected_err2 = ".*svn: Unreadable path encountered; access denied.*"
+    expected_err2 = ".*svn: E220001: Unreadable path encountered; access denied.*"
 
   svntest.actions.run_and_verify_svn(None, None, expected_err2,
                                      'cat', '-r', '2', G_url+'/rho')
@@ -584,6 +633,8 @@ def authz_log_and_tracing_test(sbox):
                                      'diff', '-r', '2:4', D_url+'/rho')
 
 # test whether read access is correctly granted and denied
+@SkipUnless(server_authz_has_aliases)
+@Skip(svntest.main.is_ra_type_file)
 def authz_aliases(sbox):
   "test authz for aliases"
 
@@ -594,7 +645,7 @@ def authz_aliases(sbox):
   if sbox.repo_url.startswith("http"):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   write_authz_file(sbox, { "/" : "* = r",
                            "/A/B" : "&jray = rw" },
@@ -620,6 +671,8 @@ def authz_aliases(sbox):
                                      '-m', 'logmsg',
                                      iota_url, B_url)
 
+@Skip(svntest.main.is_ra_type_file)
+@Issue(2486)
 def authz_validate(sbox):
   "test the authz validation rules"
 
@@ -676,14 +729,18 @@ devs1 = @admins, dev1
 devs2 = @admins, dev2
 users = @devs1, @devs2, user1, user2""" })
 
-  # validation of this authz file should fail, so no repo access
+  # validation of this authz file should *not* fail (where formerly,
+  # it complained about circular dependencies that do not, in fact,
+  # exist), so this is business as usual.
   svntest.actions.run_and_verify_svn("ls remote folder",
-                                      ['B/\n', 'C/\n', 'D/\n', 'mu\n'],
-                                      [],
+                                     ['B/\n', 'C/\n', 'D/\n', 'mu\n'],
+                                     [],
                                      'ls',
                                      A_url)
 
 # test locking/unlocking with authz
+@Skip(svntest.main.is_ra_type_file)
+@Issue(2700)
 def authz_locking(sbox):
   "test authz for locking"
 
@@ -695,7 +752,7 @@ def authz_locking(sbox):
   if sbox.repo_url.startswith('http'):
     expected_err = ".*[Ff]orbidden.*"
   else:
-    expected_err = ".*svn: Authorization failed.*"
+    expected_err = ".*svn: E170001: Authorization failed.*"
 
   root_url = sbox.repo_url
   wc_dir = sbox.wc_dir
@@ -743,6 +800,9 @@ def authz_locking(sbox):
 # test for issue #2712: if anon-access == read, svnserve should also check
 # authz to determine whether a checkout/update is actually allowed for
 # anonymous users, and, if not, attempt authentication.
+@XFail()
+@Issue(2712)
+@SkipUnless(svntest.main.is_ra_type_svn)
 def authz_svnserve_anon_access_read(sbox):
   "authz issue #2712"
 
@@ -805,6 +865,8 @@ def authz_svnserve_anon_access_read(sbox):
                                      'merge', '-c', '2',
                                      B_url, B_path)
 
+@XFail()
+@Skip(svntest.main.is_ra_type_file)
 def authz_switch_to_directory(sbox):
   "switched to directory, no read access on parents"
 
@@ -822,7 +884,54 @@ def authz_switch_to_directory(sbox):
   # Switch /A/B/E to /A/B/F.
   svntest.main.run_svn(None, 'switch', sbox.repo_url + "/A/B/E", G_path)
 
+# Test to reproduce the problem identified by Issue 3242 in which
+# Subversion's authz, as of Subversion 1.5, requires access to the
+# repository root for copy and move operations.
+@Skip(svntest.main.is_ra_type_file)
+@Issue(3242)
+def authz_access_required_at_repo_root(sbox):
+  "authz issue #3242 - access required at repo root"
 
+  sbox.build(create_wc = False)
+  root_url = sbox.repo_url
+
+  # Create a copy-level copy of A, just so we have something to work with.
+  svntest.main.run_svn(None, 'cp', '-m', 'logmsg',
+                       root_url + '/A',
+                       root_url + '/A-copy')
+
+  # Now we get all restrictive.
+  write_authz_file(sbox, {'/': '* =',
+                          '/A': 'jrandom = rw',
+                          '/A-copy': 'jrandom = rw'})
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  # Do some copies and moves where the common parents of the source(s)
+  # and destination(s) are unreadable.  All we currently hope to support
+  # is the case where the sources are individually (and recursively)
+  # readable, and the destination tree is writable.
+
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'copy in readable space',
+                       root_url + '/A/B',
+                       root_url + '/A/B-copy')
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'copy across disjoint readable spaces',
+                       root_url + '/A/B',
+                       root_url + '/A-copy/B-copy')
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'multi-copy across disjoint readable spaces',
+                       root_url + '/A/B',
+                       root_url + '/A/mu',
+                       root_url + '/A-copy/C')
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'copy from disjoint readable spaces',
+                       root_url + '/A/B/E/alpha',
+                       root_url + '/A-copy/B/E/beta',
+                       root_url + '/A-copy/C')
+
+@Skip(svntest.main.is_ra_type_file)
+@Issue(3242)
 def authz_access_required_at_repo_root2(sbox):
   "more authz issue #3242 - update to renamed file"
 
@@ -839,8 +948,8 @@ def authz_access_required_at_repo_root2(sbox):
                        '-m', 'rename file in readable writable space',
                        root_url + '/A/B/E/alpha',
                        root_url + '/A/B/E/alpha-renamed')
-  
-  # Check out original greek sub tree below /A/B/E 
+
+  # Check out original greek sub tree below /A/B/E
   # and update it to the above rename.
   wc_dir = sbox.add_wc_path('ABE')
   os.mkdir(wc_dir)
@@ -852,7 +961,7 @@ def authz_access_required_at_repo_root2(sbox):
                        '-m', 'rename diretory in readable writable space',
                        root_url + '/A/D/H',
                        root_url + '/A/D/a g e')
-  
+
   # Check out original greek sub tree below /A/D
   # and update it to the above rename.
   wc_dir = sbox.add_wc_path('AD')
@@ -860,35 +969,461 @@ def authz_access_required_at_repo_root2(sbox):
   svntest.main.run_svn(None, 'co', '-r', '1', root_url + '/A/D', wc_dir)
   svntest.main.run_svn(None, 'up', wc_dir)
 
+@Skip(svntest.main.is_ra_type_file)
+def multiple_matches(sbox):
+  "multiple lines matching a user"
+
+  sbox.build(create_wc = False)
+  root_url = sbox.repo_url
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+  if sbox.repo_url.startswith("http"):
+    expected_err = ".*[Ff]orbidden.*"
+  else:
+    expected_err = ".*svn: E170001: Authorization failed.*"
+
+  # Prohibit access and commit fails
+  write_authz_file(sbox, {'/': 'jrandom ='})
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'cp', '-m', 'fail copy',
+                                     root_url, root_url + '/fail')
+
+  # At present if multiple lines match the permissions of all the
+  # matching lines are amalgamated.  So jrandom gets access regardless
+  # of the line prohibiting access and regardless of the  order of the
+  # lines.  This might be a bug, but we probably can't simply fix it as
+  # that would change the behaviour of lots of existing authz files.
+
+  write_authz_file(sbox, {'/': 'jrandom =' + '\n' + '* = rw'})
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'first copy',
+                       root_url, root_url + '/first')
+
+  write_authz_file(sbox, {'/': '* = rw' + '\n' + 'jrandom ='})
+  svntest.main.run_svn(None, 'cp',
+                       '-m', 'second copy',
+                       root_url, root_url + '/second')
+
+@Issues(4025,4026)
+@Skip(svntest.main.is_ra_type_file)
+def wc_wc_copy_revert(sbox):
+  "wc-to-wc-copy with absent nodes and then revert"
+
+  sbox.build(create_wc = False)
+  local_dir = sbox.wc_dir
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  write_authz_file(sbox, {'/'       : '* = r',
+                          '/A/B/E'  : '* =', })
+
+  expected_output = svntest.main.greek_state.copy()
+  expected_output.wc_dir = local_dir
+  expected_output.tweak(status='A ', contents=None)
+  expected_output.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+  expected_wc = svntest.main.greek_state.copy()
+  expected_wc.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+
+  svntest.actions.run_and_verify_checkout(sbox.repo_url, local_dir,
+                                          expected_output,
+                                          expected_wc)
+
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 1)
+  expected_status.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+  svntest.actions.run_and_verify_status(sbox.wc_dir, expected_status)
+
+  svntest.actions.run_and_verify_svn(None, None,
+                             'svn: E155035: Cannot copy.*excluded by server',
+                             'cp', sbox.ospath('A'), sbox.ospath('A2'))
+
+
+  # The copy failed and A2/B/E is incomplete.  That means A2 and A2/B
+  # are complete, but for the other parts of A2 the status is undefined.
+  expected_output = svntest.verify.ExpectedOutput(
+    ['A  +             -        1 jrandom      ' + sbox.ospath('A2') + '\n',
+     '   +             -        1 jrandom      ' + sbox.ospath('A2/B') + '\n',
+     '!                -       ?   ?           ' + sbox.ospath('A2/B/E') + '\n',
+     ])
+  expected_output.match_all = False
+  svntest.actions.run_and_verify_svn(None, expected_output, [],
+                                     'st', '--verbose', sbox.ospath('A2'))
+
+
+  # Issue 4025, info SEGV on incomplete working node
+  svntest.actions.run_and_verify_svn(None, None,
+                                     'svn: E145000: .*unrecognized node kind',
+                                     'info', sbox.ospath('A2/B/E'))
+
+  # Issue 4026, copy assertion on incomplete working node
+  svntest.actions.run_and_verify_svn(None, None,
+                             'svn: E145001: cannot handle node kind',
+                             'cp', sbox.ospath('A2/B'), sbox.ospath('B3'))
+
+  expected_output = svntest.verify.ExpectedOutput(
+    ['A  +             -        1 jrandom      ' + sbox.ospath('B3') + '\n',
+     '!                -       ?   ?           ' + sbox.ospath('B3/E') + '\n',
+     ])
+  expected_output.match_all = False
+  svntest.actions.run_and_verify_svn(None, expected_output, [],
+                                     'st', '--verbose', sbox.ospath('B3'))
+
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'revert', '--recursive',
+                                     sbox.ospath('A2'), sbox.ospath('B3'))
+
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 1)
+  expected_status.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+  svntest.actions.run_and_verify_status(sbox.wc_dir, expected_status)
+
+@Skip(svntest.main.is_ra_type_file)
+def authz_recursive_ls(sbox):
+  "recursive ls with private subtrees"
+
+  sbox.build(create_wc = False)
+  local_dir = sbox.wc_dir
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  write_authz_file(sbox, {'/'       : '* = r',
+                          '/A/B/E'  : '* =',
+                          '/A/mu'   : '* =',
+                          })
+  expected_entries = [
+    'A/',
+    'A/B/',
+    'A/B/F/',
+    'A/B/lambda',
+    'A/C/',
+    'A/D/',
+    'A/D/G/',
+    'A/D/G/pi',
+    'A/D/G/rho',
+    'A/D/G/tau',
+    'A/D/H/',
+    'A/D/H/chi',
+    'A/D/H/omega',
+    'A/D/H/psi',
+    'A/D/gamma',
+    'iota',
+    ]
+  svntest.actions.run_and_verify_svn('recursive ls from /',
+                                     map(lambda x: x + '\n', expected_entries),
+                                     [], 'ls', '-R',
+                                     sbox.repo_url)
+
+@Issue(3781)
+@Skip(svntest.main.is_ra_type_file)
+def case_sensitive_authz(sbox):
+  "authz issue #3781, check case sensitiveness"
+
+  sbox.build()
+
+  wc_dir = sbox.wc_dir
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  mu_path = os.path.join(wc_dir, 'A', 'mu')
+  mu_url = sbox.repo_url + '/A/mu'
+  mu_repo_path = sbox.repo_dir + "/A/mu"
+  svntest.main.file_append(mu_path, "hi")
+
+  # Create expected output tree.
+  expected_output = svntest.wc.State(wc_dir, {
+    'A/mu' : Item(verb='Sending'),
+    })
+
+  # error messages
+  expected_error_for_commit = "Commit failed"
+
+  if sbox.repo_url.startswith("http"):
+    expected_error_for_cat = ".*[Ff]orbidden.*"
+  else:
+    expected_error_for_cat = ".*svn: E170001: Authorization failed.*"
+
+  # test the case-sensitivity of the path inside the repo
+  write_authz_file(sbox, {"/": "jrandom = r",
+                          "/A/mu": "jrandom =", "/a/Mu": "jrandom = rw"})
+  svntest.actions.run_and_verify_svn2(None, None,
+                                      expected_error_for_cat,
+                                      1, 'cat', mu_url)
+
+  write_authz_file(sbox, {"/": "jrandom = r",
+                          "/A": "jrandom = r",
+                          "/a/Mu": "jrandom = rw"})
+  # Commit the file.
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        None,
+                                        None,
+                                        expected_error_for_commit,
+                                        mu_path)
+
+  def mixcases(repo_name):
+    mixed_repo_name = ''
+    for i in range(0, len(repo_name)):
+      if i % 2 == 0:
+        mixed_val = repo_name[i].upper()
+        mixed_repo_name = mixed_repo_name + mixed_val
+      else:
+        mixed_val = repo_name[i].lower()
+        mixed_repo_name = mixed_repo_name + mixed_val
+    return mixed_repo_name
+
+  mixed_case_repo_dir = mixcases(os.path.basename(sbox.repo_dir))
+
+  # test the case-sensitivity of the repo name
+  sec_mixed_case = {mixed_case_repo_dir + ":/": "jrandom = r",
+                    mixed_case_repo_dir + ":/A": "jrandom = r",
+                    os.path.basename(sbox.repo_dir) + ":/A/mu": "jrandom =",
+                    mixed_case_repo_dir + ":/A/mu": "jrandom = rw"}
+  write_authz_file(sbox, {}, sec_mixed_case)
+  svntest.actions.run_and_verify_svn2(None, None,
+                                      expected_error_for_cat,
+                                      1, 'cat', mu_url)
+
+  write_authz_file(sbox, {},
+                   sections = {mixed_case_repo_dir + ":/": "jrandom = r",
+                               mixed_case_repo_dir + ":/A": "jrandom = r",
+                               mixed_case_repo_dir + ":/A/mu": "jrandom = rw"})
+
+  # Commit the file again.
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        None,
+                                        None,
+                                        expected_error_for_commit,
+                                        mu_path)
+
+  # test the case-sensitivity
+  write_authz_file(sbox, {"/": "jrandom = r",
+                          "/A": "jrandom = r", "/A/mu": "jrandom = rw"})
+
+  svntest.actions.run_and_verify_svn2('No error',
+                                      svntest.verify.AnyOutput, [],
+                                      0, 'cat', mu_url)
+  # Commit the file.
+  svntest.actions.run_and_verify_commit(wc_dir,
+                                        expected_output,
+                                        None,
+                                        None,
+                                        mu_path)
+
+@Skip(svntest.main.is_ra_type_file)
+def authz_tree_conflict(sbox):
+  "authz should notice a tree conflict"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  sbox.simple_rm('A/C')
+  sbox.simple_commit()
+  sbox.simple_update()
+
+  write_authz_file(sbox, {"/": "jrandom = rw", "/A/C": "*="})
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  # And now create an obstruction
+  sbox.simple_mkdir('A/C')
+
+  expected_output = svntest.wc.State(wc_dir, {})
+  expected_status = svntest.actions.get_virginal_state(wc_dir, 2)
+  expected_status.tweak('A/C', status='A ', wc_rev='0')
+
+  svntest.actions.run_and_verify_update(wc_dir,
+                                        expected_output,
+                                        None,
+                                        expected_status,
+                                        "Failed to mark '.*C' absent:",
+                                        None, None, None, None, 0,
+                                        '-r', '1', wc_dir)
+
+@Issue(3900)
+@Skip(svntest.main.is_ra_type_file)
+def wc_delete(sbox):
+  "wc delete with absent nodes"
+
+  sbox.build(create_wc = False)
+  local_dir = sbox.wc_dir
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  write_authz_file(sbox, {'/'       : '* = r',
+                          '/A/B/E'  : '* =', })
+
+  expected_output = svntest.main.greek_state.copy()
+  expected_output.wc_dir = local_dir
+  expected_output.tweak(status='A ', contents=None)
+  expected_output.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+  expected_wc = svntest.main.greek_state.copy()
+  expected_wc.remove('A/B/E', 'A/B/E/alpha', 'A/B/E/beta')
+
+  svntest.actions.run_and_verify_checkout(sbox.repo_url, local_dir,
+                                          expected_output,
+                                          expected_wc)
+
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 1)
+
+  expected_err = ".*svn: E155035: .*excluded by server*"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'rm', sbox.ospath('A/B/E'))
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'rm', sbox.ospath('A'))
+
+  expected_status = svntest.actions.get_virginal_state(sbox.wc_dir, 1)
+
+
+@Skip(svntest.main.is_ra_type_file)
+def wc_commit_error_handling(sbox):
+  "verify commit error reporting"
+
+  sbox.build()
+  wc_dir = sbox.wc_dir
+  write_restrictive_svnserve_conf(sbox.repo_dir)
+
+  sbox.simple_mkdir('A/Z')
+
+  write_authz_file(sbox, {'/'   : '* = r', })
+
+  # Creating editor fail: unfriendly error
+  expected_err = "(svn: E175013: .*orbidden.*)|" + \
+                 "(svn: E170001: Authorization failed)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+  write_authz_file(sbox, {'/'   : '* = rw',
+                          '/A'  : '* = r', })
+
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing directory '.*Z' is forbidden)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+  sbox.simple_revert('A/Z')
+
+  svntest.main.file_write(sbox.ospath('A/zeta'), "Zeta")
+  sbox.simple_add('A/zeta')
+
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing file '.*zeta' is forbidden)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+  sbox.simple_revert('A/zeta')
+
+  sbox.simple_propset('a', 'b', 'A/D')
+
+  # Allow a generic dav error and the ra_svn specific one that is returned
+  # on editor->edit_close().
+  expected_err = "(svn: E175013: .*orbidden.*)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+  sbox.simple_revert('A/D')
+
+  sbox.simple_propset('a', 'b', 'A/B/lambda')
+
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing file '.*lambda' is forbidden.*)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+  sbox.simple_revert('A/B/lambda')
+
+  svntest.main.file_write(sbox.ospath('A/B/lambda'), "New lambda")
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing file '.*lambda' is forbidden.*)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+  sbox.simple_revert('A/B/lambda')
+
+  sbox.simple_rm('A/B/F')
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing directory '.*F' is forbidden.*)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+  sbox.simple_revert('A/B/F')
+
+  svntest.main.file_write(sbox.ospath('A/mu'), "Updated mu")
+  # Allow the informative error for dav and the ra_svn specific one that is
+  # returned on editor->edit_close().
+  expected_err = "(svn: E195023: Changing file '.*mu' is forbidden.*)|" + \
+                 "(svn: E220004: Access denied)"
+  svntest.actions.run_and_verify_svn(None, None, expected_err,
+                                     'ci', wc_dir, '-m', '')
+
+
+@Skip(svntest.main.is_ra_type_file)
+def upgrade_absent(sbox):
+  "upgrade absent nodes to server-excluded"
+
+  # Install wc and repos
+  replace_sbox_with_tarfile(sbox, 'upgrade_absent.tar.bz2')
+  replace_sbox_repo_with_tarfile(sbox, 'upgrade_absent_repos.tar.bz2')
+
+  # Update config for authz
+  svntest.main.write_restrictive_svnserve_conf(sbox.repo_dir)
+  svntest.main.write_authz_file(sbox, { "/"      : "*=rw",
+                                        "/A/B"   : "*=",
+                                        "/A/B/E" : "jrandom = rw"})
+
+  # Attempt to use the working copy, this should give an error
+  expected_stderr = wc_is_too_old_regex
+  svntest.actions.run_and_verify_svn(None, None, expected_stderr,
+                                     'info', sbox.wc_dir)
+
+  # Now upgrade the working copy
+  svntest.actions.run_and_verify_svn(None, None, [],
+                                     'upgrade', sbox.wc_dir)
+
+  # Relocate to allow finding the repository
+  svntest.actions.run_and_verify_svn(None, None, [], 'relocate',
+                                     'svn://127.0.0.1/authz_tests-2',
+                                     sbox.repo_url, sbox.wc_dir)  
+
+  expected_output = svntest.wc.State(sbox.wc_dir, {
+  })
+
+  # Expect no changes and certainly no errors
+  svntest.actions.run_and_verify_update(sbox.wc_dir, expected_output,
+                                        None, None)
+
 ########################################################################
 # Run the tests
 
 # list all tests here, starting with None:
 test_list = [ None,
-              Skip(authz_open_root, svntest.main.is_ra_type_file),
-              Skip(authz_open_directory, svntest.main.is_ra_type_file),
-              Skip(broken_authz_file, svntest.main.is_ra_type_file),
-              Skip(authz_read_access, svntest.main.is_ra_type_file),
-              Skip(authz_write_access, svntest.main.is_ra_type_file),
-              Skip(authz_checkout_test, svntest.main.is_ra_type_file),
-              Skip(authz_log_and_tracing_test, svntest.main.is_ra_type_file),
-              Skip(authz_checkout_and_update_test,
-                   svntest.main.is_ra_type_file),
-              Skip(authz_partial_export_test, svntest.main.is_ra_type_file),
-              SkipUnless(Skip(authz_aliases, svntest.main.is_ra_type_file),
-                         server_authz_has_aliases),
-              Skip(authz_validate, svntest.main.is_ra_type_file),
-              Skip(authz_locking, svntest.main.is_ra_type_file),
-              XFail(SkipUnless(authz_svnserve_anon_access_read,
-                               svntest.main.is_ra_type_svn)),
-              XFail(Skip(authz_switch_to_directory,
-                         svntest.main.is_ra_type_file)),
-              Skip(authz_access_required_at_repo_root2,
-                   svntest.main.is_ra_type_file),
+              authz_open_root,
+              authz_open_directory,
+              broken_authz_file,
+              authz_read_access,
+              authz_write_access,
+              authz_checkout_test,
+              authz_log_and_tracing_test,
+              authz_checkout_and_update_test,
+              authz_partial_export_test,
+              authz_aliases,
+              authz_validate,
+              authz_locking,
+              authz_svnserve_anon_access_read,
+              authz_switch_to_directory,
+              authz_access_required_at_repo_root,
+              authz_access_required_at_repo_root2,
+              multiple_matches,
+              wc_wc_copy_revert,
+              authz_recursive_ls,
+              case_sensitive_authz,
+              authz_tree_conflict,
+              wc_delete,
+              wc_commit_error_handling,
+              upgrade_absent,
              ]
+serial_only = True
 
 if __name__ == '__main__':
-  svntest.main.run_tests(test_list, serial_only = True)
+  svntest.main.run_tests(test_list, serial_only = serial_only)
   # NOTREACHED
 
 
